<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 9:16 PM
 */

namespace qfq;

use qfq;

//use qfq\exceptions\CodeException;

require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/Constants.php');
require_once(__DIR__ . '/../../qfq/exceptions/CodeException.php');

use PHPUnit\Framework\TestCase;

//class StoreTest extends \PHPUnit_Framework_TestCase {

/**
 * Class StoreTest
 * @package qfq
 */
class StoreTest extends TestCase {

    /**
     * @var Store
     */
    private $store = null;

    private $setUpDone = false;

    public function setUp() {
        // Client Variables has to setup before the first instantiation of 'Store'
        $_GET[CLIENT_RECORD_ID] = '1234';
//        $_GET[CLIENT_SIP] = '12badcaffee34';
        $_GET['key01'] = '1234';
        $_POST['key02'] = '2345';
        $_POST['key03'] = '3456';
        $_POST[CLIENT_FORM] = 'testformnameDoNotChange';
        $_GET['key04'] = '4567';
        $_POST['key04'] = '5678';

        $this->store = Store::getInstance("\n; some comment\n" . TYPO3_FORM . "=testformnameDoNotChange\n", true);

    }

    public function testGetInstance() {
        $a = Store::getInstance('', true);
        $b = Store::getInstance('', true);

        $this->assertFalse($a === null, "There should be a class");

        $this->assertEquals($a, $b, "Both classes should be the same");
    }

    public function testGetVarStoreTypo3() {
        // T3 Bodytext
        $this->store->setVar(TYPO3_FORM, "testformnameDoNotChange", STORE_TYPO3);
        $this->assertEquals('testformnameDoNotChange', $this->store->getVar(TYPO3_FORM, STORE_TYPO3), "System: " . TYPO3_FORM);

    }

    public function testGetVarStoreSystem() {
        // DBUSER in qfq.ini
        $val = $this->store->getVar(SYSTEM_DB_INDEX_DATA, STORE_SYSTEM);
        $this->assertTrue($val !== false && $val !== '', "SYSTEM_DB_INDEX_DATA found in qfq.ini");

    }

    public function testSetVarStoreSystem() {

        // set new Sessionname
        $this->store->setVar(SYSTEM_SQL_LOG_MODE, "all", STORE_SYSTEM);

        // Sessionname: default value
        $this->assertEquals('all', $this->store->getVar(SYSTEM_SQL_LOG_MODE, STORE_SYSTEM), "System: SQL_LOG");

        // set new Sessionname
        $this->store->setVar(SYSTEM_SQL_LOG_MODE, "modify", STORE_SYSTEM);

        $this->assertEquals('modify', $this->store->getVar(SYSTEM_SQL_LOG_MODE, STORE_SYSTEM), "System: SQL_LOG");

    }


    public function testGetVarStoreClient() {

        # Violates SANITIZE class: sanitized string is always an empty string.
        # Access are cached:

        # Test: Retrieve a variable, default sanitize class
        $this->assertEquals('1234', $this->store->getVar(CLIENT_RECORD_ID, STORE_CLIENT), "FormName: default sanitize class");

        # violates default SANITIZE digit: sanitized string is always an empty string.
        $this->assertEquals('', $this->store->getVar(CLIENT_SIP, STORE_CLIENT), "sanitize class digit fails");

        // Test GET
        $this->assertEquals('1234', $this->store->getVar('key01', STORE_CLIENT), "Param: GET");

        // Test POST
        $this->assertEquals('2345', $this->store->getVar('key02', STORE_CLIENT), "Param: POST");

        // Test not found
        $this->assertFalse($this->store->getVar('keyUnknown', STORE_CLIENT), "Param: unknown");

        // Test Cache value
        $_POST['key03'] = 'lost';
        $this->store->getVar('key03', STORE_CLIENT);
        $this->assertEquals('3456', $this->store->getVar('key03', STORE_CLIENT), "Param: read from cache");

        // Test Cache not found
        $this->store->getVar('keyUnknwon2');
        $this->assertFalse($this->store->getVar('keyUnknown2', STORE_CLIENT), "Param: unknown from cache");

        // Test overwrite default sanitize class
        $this->assertEquals('!!digit!!', $this->store->getVar(CLIENT_FORM, STORE_CLIENT, SANITIZE_ALLOW_DIGIT), "Param: overwrite default sanitize class");

        // Test: POST higher priority than GET
        $this->assertEquals('5678', $this->store->getVar('key04', STORE_CLIENT), "Param: POST higher priority than GET");
    }


    public function testStorePriority() {
        //default prio  FSRVD

        $this->store->unsetStore(STORE_RECORD);

        $db = new Database();
        $tableDefinition = $db->getTableDefinition('Person');
        $this->store->fillStoreTableDefaultColumnType($tableDefinition);

//        $this->assertEquals('male', $this->store->getVar('gender','','alnumx'), "Get default definition from table person.gender");

        $this->store->setVar('gender', 'female', STORE_RECORD);
        $this->assertEquals('female', $this->store->getVar('gender'), "Retrieve 'gender' from STORE_RECORD");

        $this->store->setVar('gender', 'male2', STORE_SIP);
        $this->assertEquals('male2', $this->store->getVar('gender'), "Retrieve 'gender' from STORE_SIP");

        $this->store->setVar('gender', 'female2', STORE_FORM);
        $this->assertEquals('female2', $this->store->getVar('gender', '', SANITIZE_ALLOW_ALNUMX), "Retrieve 'gender' from STORE_SIP");

    }

    public function testStoreDifferentSanitizeClass() {
        //default prio  FSRD

        $this->store->setVar('fruit', 'apple', STORE_RECORD);
        $this->assertEquals('apple', $this->store->getVar('fruit'), "Retrieve 'fruit' from STORE_RECORD");

        $this->store->setVar('color', 'green', STORE_FORM);
        $this->assertEquals('!!digit!!', $this->store->getVar('color'), "Retrieve 'color' from STORE_FORM");
        $this->assertEquals('green', $this->store->getVar('color', '', SANITIZE_ALLOW_ALNUMX), "Retrieve 'color' from STORE_FORM");
    }

    public function testGetVarStore0() {
        //default prio  FSRD

        $this->assertEquals(0, $this->store->getVar('fakename', STORE_ZERO), "Retrieve anything from STORE_ZERO");
    }

    public function testUnsetStore() {
        $this->store->unsetStore(STORE_RECORD);

        $this->store->setStore(array(), STORE_RECORD);
        $this->assertEquals(false, $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }

    public function testSetVarArray() {
        $this->store->unsetStore(STORE_RECORD);
        $arr = ['a' => '1', 'apple' => 'green'];
        $this->store->setStore($arr, STORE_RECORD);
        $this->assertEquals('1', $this->store->getVar('a', STORE_RECORD), "Retrieve a value from store 'fake'");
        $this->assertEquals('green', $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }


    public function testSetVarArrayEmpty() {
        $this->store->unsetStore(STORE_RECORD);
        $arr = array();
        $this->store->setStore($arr, STORE_RECORD);
        $this->assertEquals(false, $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }

    public function testSetVar() {

        $this->store = Store::getInstance('', true);

        foreach ([STORE_SIP, STORE_CLIENT, STORE_EXTRA] AS $store) {
            // initial set
            $this->store->setVar('unitTest', '123', $store);
            $this->assertEquals('123', $this->store->getVar('unitTest', $store), "Retrieve var.");

            // redefine, using default 'overwrite=true'
            $this->store->setVar('unitTest', '1234', $store);
            $this->assertEquals('1234', $this->store->getVar('unitTest', $store), "Retrieve var.");

            // redefine, using explicit 'overwrite=true'
            $this->store->setVar('unitTest', '12345', $store, true);
            $this->assertEquals('12345', $this->store->getVar('unitTest', $store), "Retrieve var.");

            // redefine, using explicit 'overwrite=false' but with the same value (that is ok)
            $this->store->setVar('unitTest', '12345', $store, false);
            $this->assertEquals('12345', $this->store->getVar('unitTest', $store), "Retrieve var.");
        }
    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testConfigMandatoryValues() {

        $fileName = $this->createFile('');
        $this->store = Store::getInstance('', true, $fileName);
        unlink($fileName);

        $this->assertEquals(false, $this->store->getStore(STORE_SYSTEM), "Retrieve system store.");
    }

    /**
     *
     */
    public function testAppendToStore() {
        $this->store->unsetStore(STORE_RECORD);

        $this->store->appendToStore(array(), STORE_RECORD);
        $this->assertEquals(array(), $this->store->getStore(STORE_RECORD));

        $param = ['a' => 'hello', 'b' => 'world'];
        $this->store->appendToStore($param, STORE_RECORD);
        $this->assertEquals($param, $this->store->getStore(STORE_RECORD));

        $this->store->appendToStore(null, STORE_RECORD);
        $this->assertEquals($param, $this->store->getStore(STORE_RECORD));

        $this->store->appendToStore(array(), STORE_RECORD);
        $this->assertEquals($param, $this->store->getStore(STORE_RECORD));

        $param2 = ['c' => 'nice', 'd' => 'job'];
        $this->store->appendToStore($param2, STORE_RECORD);
        $this->assertEquals(array_merge($param, $param2), $this->store->getStore(STORE_RECORD));

        $param3 = ['a' => 'well', 'b' => 'done'];
        $this->store->appendToStore($param3, STORE_RECORD);
        $this->assertEquals(array_merge($param3, $param2), $this->store->getStore(STORE_RECORD));

        $this->store->setStore($param, STORE_RECORD, true);
        $this->store->appendToStore([$param2, $param3, $param3], STORE_RECORD);
        $this->assertEquals(array_merge($param, $param2), $this->store->getStore(STORE_RECORD));
    }

    /**
     * @param $body
     *
     * @return string
     */
    private function createFile($body) {

        $tmpFileName = tempnam('/tmp', 'config.qfq.ini.');
        $handle = fopen($tmpFileName, "w");
        fwrite($handle, $body);
        fclose($handle);

        return $tmpFileName;
    }

    public function testConfigIniDefaultValues() {

        $body = <<< EOT
                ; Comment
                DB_1_USER = <DBUSER>
                DB_1_SERVER = <DBSERVER>
                DB_1_PASSWORD = <DBPW>
                DB_1_NAME = <DB>

                DB_INIT = set names utf8

                SQL_LOG = sql.log
                ; all, modify
                SQL_LOG_MODE = modify
EOT;

        $expect = [
            SYSTEM_DB_INDEX_DATA => '1',
            SYSTEM_DB_INDEX_QFQ => '1',

            SYSTEM_DATE_FORMAT => 'yyyy-mm-dd',
            SYSTEM_SHOW_DEBUG_INFO => SYSTEM_SHOW_DEBUG_INFO_AUTO,

            SYSTEM_SQL_LOG_MODE => 'modify',

            F_BS_COLUMNS => '12',
            F_BS_LABEL_COLUMNS => '3',
            F_BS_INPUT_COLUMNS => '6',
            F_BS_NOTE_COLUMNS => '3',

            SYSTEM_CMD_WKHTMLTOPDF => '/opt/wkhtmltox/bin/wkhtmltopdf',

            F_CLASS_PILL => 'qfq-color-grey-1',
            F_CLASS_BODY => 'qfq-color-grey-2',

            F_SAVE_BUTTON_TEXT => '',
            F_SAVE_BUTTON_TOOLTIP => 'Save',
            F_SAVE_BUTTON_CLASS => 'btn btn-default navbar-btn',
            F_SAVE_BUTTON_GLYPH_ICON => 'glyphicon-ok',

            F_CLOSE_BUTTON_TEXT => '',
            F_CLOSE_BUTTON_TOOLTIP => 'Close',
            F_CLOSE_BUTTON_CLASS => 'btn btn-default navbar-btn',
            F_CLOSE_BUTTON_GLYPH_ICON => 'glyphicon-remove',

            F_DELETE_BUTTON_TEXT => '',
            F_DELETE_BUTTON_TOOLTIP => 'Delete',
            F_DELETE_BUTTON_CLASS => 'btn btn-default navbar-btn',
            F_DELETE_BUTTON_GLYPH_ICON => 'glyphicon-trash',

            F_NEW_BUTTON_TEXT => '',
            F_NEW_BUTTON_TOOLTIP => 'New',
            F_NEW_BUTTON_CLASS => 'btn btn-default navbar-btn',
            F_NEW_BUTTON_GLYPH_ICON => 'glyphicon-plus',

            F_BUTTON_ON_CHANGE_CLASS => 'btn-info alert-info',
            SYSTEM_EDIT_FORM_PAGE => 'form',
            SYSTEM_SECURITY_VARS_HONEYPOT => 'email,username,password',
            SYSTEM_SECURITY_ATTACK_DELAY => 5,
            SYSTEM_SECURITY_SHOW_MESSAGE => '0',
            SYSTEM_SECURITY_GET_MAX_LENGTH => 50,
            SYSTEM_ESCAPE_TYPE_DEFAULT => 'm',
            SYSTEM_EXTRA_BUTTON_INFO_INLINE => '<span class="glyphicon glyphicon-info-sign" aria-hidden="true"></span>',
            SYSTEM_EXTRA_BUTTON_INFO_BELOW => '<span class="glyphicon glyphicon-info-sign text-info" aria-hidden="true"></span>',
            SYSTEM_EXTRA_BUTTON_INFO_CLASS => '',

            SYSTEM_DB_UPDATE => SYSTEM_DB_UPDATE_AUTO,
            SYSTEM_RECORD_LOCK_TIMEOUT_SECONDS => SYSTEM_RECORD_LOCK_TIMEOUT_SECONDS_DEFAULT,

            DOCUMENTATION_QFQ => DOCUMENTATION_QFQ_URL,
            SYSTEM_ENTER_AS_SUBMIT => 1,
            SYSTEM_CMD_INKSCAPE => 'inkscape',
            SYSTEM_CMD_CONVERT => 'convert',
            SYSTEM_THUMBNAIL_DIR_SECURE => SYSTEM_THUMBNAIL_DIR_SECURE_DEFAULT,
            SYSTEM_THUMBNAIL_DIR_PUBLIC => SYSTEM_THUMBNAIL_DIR_PUBLIC_DEFAULT,

            SYSTEM_DB_1_USER => '<DBUSER>',
            SYSTEM_DB_1_SERVER => '<DBSERVER>',
            SYSTEM_DB_1_PASSWORD => '<DBPW>',
            SYSTEM_DB_1_NAME => '<DB>',

            SYSTEM_DB_INIT => 'set names utf8',

        ];

        $fileName = $this->createFile($body);
        $this->store = Store::getInstance('', true, $fileName);
        unlink($fileName);

        $value = $this->store->getStore(STORE_SYSTEM);
        // these values are different between runtime and PHPUnit environment: do not check
        unset($value[SYSTEM_SQL_LOG]);
        unset($value[SYSTEM_PATH_EXT]);
        unset($value[SYSTEM_SITE_PATH]);
        unset($value[SYSTEM_MAIL_LOG]);
        unset($value[SYSTEM_SEND_E_MAIL]);

        // check default values
        $this->assertEquals($expect, $value, "Retrieve system store.");

        //
        $body .= PHP_EOL . SYSTEM_FORM_BS_LABEL_COLUMNS . ' = 4';
        $body .= PHP_EOL . SYSTEM_FORM_BS_INPUT_COLUMNS . ' = 5';
        $body .= PHP_EOL . SYSTEM_FORM_BS_NOTE_COLUMNS . ' = 6';
        $body .= PHP_EOL . SYSTEM_FORM_DATA_PATTERN_ERROR . ' = Pattern error';
        $body .= PHP_EOL . SYSTEM_FORM_DATA_REQUIRED_ERROR . ' = Required error';
        $body .= PHP_EOL . SYSTEM_FORM_DATA_MATCH_ERROR . ' = Fields do not match';
        $body .= PHP_EOL . SYSTEM_FORM_DATA_ERROR . ' = Error';
        $body .= PHP_EOL . SYSTEM_CSS_CLASS_QFQ_FORM . ' = main-class';
        $body .= PHP_EOL . SYSTEM_CSS_CLASS_QFQ_FORM_PILL . ' = pill-class';
        $body .= PHP_EOL . SYSTEM_CSS_CLASS_QFQ_FORM_BODY . ' = body-class';

        $expect[F_BS_LABEL_COLUMNS] = '4';
        $expect[F_BS_INPUT_COLUMNS] = '5';
        $expect[F_BS_NOTE_COLUMNS] = '6';
        $expect[F_FE_DATA_PATTERN_ERROR] = 'Pattern error';
        $expect[F_FE_DATA_REQUIRED_ERROR] = 'Required error';
        $expect[F_FE_DATA_MATCH_ERROR] = 'Fields do not match error';
        $expect[F_FE_DATA_ERROR] = 'Error';
        $expect[F_CLASS] = 'main-class';
        $expect[F_CLASS_PILL] = 'pill-class';
        $expect[F_CLASS_BODY] = 'body-class';

        $fileName = $this->createFile($body);
        $this->store = Store::getInstance('', true, $fileName);
        unlink($fileName);

        $value = $this->store->getStore(STORE_SYSTEM);
        // these values are different between runtime and PHPUnit environment: do not check
        unset($value[SYSTEM_SQL_LOG]);
        unset($value[SYSTEM_PATH_EXT]);
        unset($value[SYSTEM_SITE_PATH]);
        unset($value[SYSTEM_MAIL_LOG]);
        unset($value[SYSTEM_SEND_E_MAIL]);

        // check default values
        $this->assertEquals($expect, $value, "Check explizit defined values.");
    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testGetStore() {
        $this->assertEquals(array(), $this->store->getStore('unknownstore'));
    }

}
